package in.slanglabs.assistants.retail;


import static io.flutter.view.FlutterMain.getLookupKeyForAsset;

import android.content.Context;
import android.content.res.AssetFileDescriptor;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.view.View;
import android.widget.ImageView;

import androidx.annotation.Nullable;

import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;
import java.net.URL;
import java.util.Map;

import io.flutter.plugin.platform.PlatformView;

public class SlangRetailAssistantConvaTriggerView implements PlatformView {

    public static SlangConvaTrigger convaTrigger;
    private Context context;

    public SlangRetailAssistantConvaTriggerView(Context context, Object obj, int id) {
        this.convaTrigger = new SlangConvaTrigger(context);
        this.context = context;

        String imageUrl = ((Map<String, Object>) obj).get("imageUrl").toString();
        String imageAsset = ((Map<String, Object>) obj).get("imageAsset").toString();

        if (!imageUrl.equalsIgnoreCase("default_image_url")) {
            setCustomImage(imageUrl);
        } else if (!imageAsset.equalsIgnoreCase("default_image_asset")) {
            setAssetImage(imageAsset);
        }
    }

    @Nullable
    @Override
    public View getView() {
        return convaTrigger;
    }

    @Override
    public void dispose() {
        convaTrigger.onDetachedFromWindow();
    }

    private void setAssetImage(String imageAsset) {
        try {
            new SampleAsync(convaTrigger, imageAssets -> {
                try {
                    AssetManager assetManager = context.getAssets();
                    String key = getLookupKeyForAsset(imageAssets);
                    AssetFileDescriptor assetFileDescriptor = assetManager.openFd(key);
                    InputStream inputStream = assetFileDescriptor.createInputStream();
                    Bitmap bitmap = BitmapFactory.decodeStream(inputStream);
                    inputStream.close();
                    return bitmap;
                } catch (IOException e) {
                    e.printStackTrace();
                }
                return null;
            }, imageAsset).execute();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    private void setCustomImage(String uriString) {
        try {
            // Debug
            if (uriString.startsWith("http")) {
                new SampleAsync(convaTrigger, urlString -> {
                    try {
                        URL url = new URL(urlString);
                        return BitmapFactory.decodeStream(url.openConnection().getInputStream());
                    } catch (IOException e) {
                    }
                    return null;
                }, uriString).execute();
            } else {
                // Release
                new SampleAsync(convaTrigger, urlString -> {
                    try {
                        int resourceId = context.getResources().getIdentifier(
                                uriString, "drawable", context.getPackageName()
                        );
                        return BitmapFactory.decodeResource(
                                context.getResources(),
                                resourceId
                        );
                    } catch (Exception e) {
                    }
                    return null;
                }, uriString).execute();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private interface BitmapDownloadInterface {
        Bitmap downloadBitmapImage(String imageString);
    }

    private static class SampleAsync extends AsyncTask<Void, Void, Bitmap> {

        private WeakReference<ImageView> mImageView;
        private BitmapDownloadInterface mBitmapDownloadInterface;
        private String mImageString;

        SampleAsync(ImageView imageView, BitmapDownloadInterface bitmapDownloadInterface, String imageString) {
            mImageView = new WeakReference<>(imageView);
            mBitmapDownloadInterface = bitmapDownloadInterface;
            mImageString = imageString;
        }

        @Override
        protected Bitmap doInBackground(Void... params) {
            try {
                return mBitmapDownloadInterface.downloadBitmapImage(mImageString);
            } catch (Exception e) {
                e.printStackTrace();
            }
            return null;
        }

        @Override
        protected void onPostExecute(Bitmap bmp) {
            super.onPostExecute(bmp);
            if (bmp != null) {
                if (mImageView != null && mImageView.get() != null) {
                    mImageView.get().setImageBitmap(bmp);
                }
            }
        }
    }
}
